from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from .widgets import MapInput, AddressAutoHiddenInput


def parse_location(location_string):
    """parse and convert coordinates from string to tuple"""

    args = location_string.split(",")
    if len(args) != 2:
        raise ValidationError(_("Invalid input for a Location instance"))

    lat = args[0]
    lng = args[1]

    try:
        lat = float(lat)
    except ValueError:
        raise ValidationError(_("Invalid input for a Location instance. Latitude must be convertible to float "))
    try:
        lng = float(lng)
    except ValueError:
        raise ValidationError(_("Invalid input for a Location instance. Longitude must be convertible to float "))

    return lat, lng


class LocationField(forms.CharField):
    """custom form field for picking location"""

    def __init__(self, *args, **kwargs):
        map_attrs = kwargs.pop("map_attrs", None)
        self.widget = MapInput(map_attrs=map_attrs, )

        super().__init__(*args, **kwargs)
        self.error_messages = {"required": "Please pick a location, it's required", }


class AddressAutoHiddenField(forms.CharField):
    """custom form field which uses AddressAutoHiddenInput"""
    widget = AddressAutoHiddenInput()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.label = ""





from django.contrib.gis.db.models import PointField
from django.utils.translation import ugettext_lazy as _

from .forms import SpatialLocationField as SpatialLocationFormField


class SpatialLocationField(PointField):
    """custom model field for storing location in spatial databases"""

    description = _("Location field for spatial databases, stores Points.")

    def __init__(self, *args, **kwargs):
        self.map_attrs = kwargs.pop("map_attrs", {})
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        kwargs["map_attrs"] = self.map_attrs
        return name, path, args, kwargs

    def formfield(self, **kwargs):
        defaults = {'form_class': SpatialLocationFormField}
        defaults.update(kwargs)
        defaults.update({"map_attrs": self.map_attrs})
        return super().formfield(**defaults)



from .models import SpatialLocationField

from ..admin import MapAdmin
from ..models import LocationField, AddressAutoHiddenField
from ..widgets import MapAdminInput, AddressAutoHiddenInput


class SpatialMapAdmin(MapAdmin):
    formfield_overrides = {
        LocationField: {'widget': MapAdminInput},
        SpatialLocationField: {'widget': MapAdminInput},
        AddressAutoHiddenField: {"widget": AddressAutoHiddenInput, }
    }
